import Data.List
import Data.List.Split
import qualified Data.Set as Set

-- represents a single point on the grid
type Position = (Int, Int)

-- represents a motion, ie U53
data Motion = Motion Char Int deriving Show

-- from a point, return a set of points generated by a motion
trace :: Position -> Motion -> [Position]
trace (sx, sy) (Motion 'U' distance) = [ (sx, sy + dy) | dy <- [1 .. distance] ]
trace (sx, sy) (Motion 'D' distance) = [ (sx, sy - dy) | dy <- [1 .. distance] ]
trace (sx, sy) (Motion 'R' distance) = [ (sx + dx, sy) | dx <- [1 .. distance] ]
trace (sx, sy) (Motion 'L' distance) = [ (sx - dx, sy) | dx <- [1 .. distance] ]

-- from a set of motions, generate an entire set of points representing a wire
-- assumes and origin of 0, 0
route :: [Motion] -> [Position]
route motions = foldl (\position motion -> position ++ (trace (last position) motion)) [(0, 0)] motions

-- get the manhattan distance between two points
manhattan :: Position -> Position -> Int
manhattan (x1, y1) (x2, y2) = (abs $ x1 - x2) + (abs $ y1 - y2)

-- get the intersections of multiple wires
crossings :: [[Motion]] -> [Position]
crossings instructions = Set.toList $ foldr1 Set.intersection $ map (\x -> Set.fromList $ route x) instructions

-- get closest crossing. the zeroth element is actually 0, 0 so we can the first
closestCrossing :: [Position] -> Position
closestCrossing positions = (sortBy (\a b -> compare (manhattan (0, 0) a) (manhattan (0, 0) b)) positions) !! 1

main = do
  -- here, the file is being broken into lines, then each line is made into a set of motions by readWire
  instructions <- readFile "input" >>= (\x -> pure $ map (readWire) $ lines x)
  putStrLn $ show $ manhattan (0, 0) (closestCrossing $ crossings instructions)

  -- readWire splits a line on its commas, then reads in each element a a Motion
  where readWire definition = map (\x -> Motion (head x) (read $ tail x)) $ splitOn "," definition
